from eazynlp.base.transcript import Transcript


class VideoTranscript(Transcript):
    """
    Child class of transcript for video transcript objects
    """

    def __init__(self, timestep_interval=None, transcript_filepath=None, metadata_filepath=None):
        super(VideoTranscript, self).__init__(transcript_filepath, metadata_filepath)
        self.timestep_interval = timestep_interval
        if timestep_interval:
            self.timestep_interval = self.get_timestep_interval()

    def get_timestep_interval(self):
        df_metadata = self.load_df_metadata()
        timestep_interval = df_metadata["Timestep Interval"]
        return timestep_interval[0]


if __name__ == "__main__":
    print("All good!")



"""
Cleaner module
Clean text from your data files into a format that allows for analysis
"""

from eazynlp.base.transcript import Transcript


class Cleaner:
    """
    Superclass that defines basic properties and methods of pipelines
    """

    def __init__(self, transcript: Transcript = None):
        """
        :param transcript: Trancript object
        """
        self.transcript = transcript

    def set_transcript(self, transcript: Transcript):
        """
        :param transcript: Trancript object
        :return: N/A
        """
        self.transcript = transcript

    def execute(self, option_dict=None):
        """
        Shell method for executing cleaning in child class
        :param option_dict: Dictionary of options that cleaners take
        :return: N/A
        """
        # if option_dict is None:
        #    option_dict = {}
        pass


if __name__ == "__main__":
    print("All is good!")



from eazynlp.base.transcript import Transcript


class Analysis:
    """
    Superclass that defines the basic properties and methods
    supported by child classes that implement specific analyses.
    """
    analysis_result = None

    def __init__(self, transcript: Transcript = None):
        """
        :param transcript: Trancript object
        """
        self.transcript = transcript

    def generate_analysis_result(self):
        """
        Shell method for generating analysis result in child class
        """
        pass

    def save_analysis_result(self, target_filepath):
        """
        Shell method for saving analysis result in child class
        to a file
        :param target_filepath: Target filepath to save analysis result
        """
        pass


if __name__ == "__main__":
    print("All good!")



"""
Helper function module for running tests
"""

import os
import pandas as pd

current_dir = os.path.dirname(os.getcwd()) + "\\tests"


def get_sample_data_filepath(datatype="interview"):
    """
    :param datatype: Can be "interview" or "video"
    :return: Sample filepath as string
    """
    if datatype == "interview":
        return current_dir+"\\test_interview_transcript.csv"
    else:
        return current_dir+"\\test_video_transcript.csv"


def get_sample_metadata_filepath(datatype="interview"):
    """
        :param datatype: Can be "interview" or "video"
        :return: Sample filepath as string
        """
    if datatype == "interview":
        return current_dir + "\\test_interview_metadata.csv"
    else:
        return current_dir + "\\test_video_metadata.csv"


def load_sample_csv(filetype="interview"):
    """
    :param filetype: Can be "interview" or "video"
    :return: Test sample csv as dataframe
    """
    # transcript = ""
    if filetype == "interview":
        transcript = "test_interview_transcript.csv"
    else:
        transcript = "test_video_transcript.csv"
    return pd.read_csv(transcript)


if __name__ == "__main__":
    print("All good!")



import pandas as pd


class Transcript:
    """
    Superclass that defines the basic properties and methods
    supported by child classes that implement specific transcript types.
    """
    def __init__(self, transcript_filepath=None, metadata_filepath=None):
        self.transcript_filepath = transcript_filepath
        self.metadata_filepath = metadata_filepath

        self.df_transcript = None
        self.df_metadata = None
        if transcript_filepath:
            self.df_transcript = pd.read_csv(transcript_filepath)
        if metadata_filepath:
            self.df_metadata = pd.read_csv(metadata_filepath)

    # Instance method
    def set_transcript_filepath(self, transcript_filepath):
        """
        Sets transcript property transcript_filepath
        :param transcript_filepath: Transcript filepath
        :return: None
        """
        self.transcript_filepath = transcript_filepath

    def set_metadata_filepath(self, metadata_filepath):
        """
        Sets transcript property metadata_filepath
        :param metadata_filepath: Metadata filepath
        :return: None
        """
        self.metadata_filepath = metadata_filepath

    def set_df_transcript(self):
        if self.transcript_filepath:
            self.df_transcript = pd.read_csv(self.transcript_filepath)

    def set_df_metadata(self):
        if self.metadata_filepath:
            self.df_metadata = pd.read_csv(self.metadata_filepath)

    def load_df_text(self):
        df_text = pd.read_csv(self.transcript_filepath)
        return df_text

    def load_df_metadata(self):
        df_metadata = pd.read_csv(self.metadata_filepath)
        return df_metadata

    def get_researcher(self):
        df_metadata = self.load_df_metadata()
        researcher = df_metadata["Researcher"]
        return researcher

    def get_participant(self):
        df_metadata = self.load_df_metadata()
        participant = df_metadata["Participant"]
        return participant

    def get_duration(self):
        df_metadata = self.load_df_metadata()
        duration = df_metadata["Duration"]
        return duration

    def get_language(self):
        df_metadata = self.load_df_metadata()
        language = df_metadata["Language"]
        return language

    def get_timestep_interval(self):
        pass


if __name__ == "__main__":
    print("All good!")



from eazynlp.base.transcript import Transcript
from eazynlp.text_cleaner.cleaner import Cleaner
from nltk.tokenize import RegexpTokenizer


class PunctuationCleaner(Cleaner):
    """
    A subclass of Cleaner superclass that removes punctuation from transcript
    """

    def execute(self, option_dict=None):
        """
        :param option_dict: An empty dictionary (no options for this cleaner subclass)
        :return:
        """
        if self.transcript.df_transcript is not None:
            tokenizer = RegexpTokenizer(r'\w+')
            self.transcript.df_transcript["Transcript"] = self.transcript.df_transcript["Transcript"].apply(
                lambda x: " ".join(str(item) for item in tokenizer.tokenize(x)))


if __name__ == "__main__":
    print("All is good!")



from eazynlp.base.transcript import Transcript
from eazynlp.text_cleaner.cleaner import Cleaner
from nltk.corpus import stopwords


class StopwordCleaner(Cleaner):
    """
    A subclass of Cleaner superclass that removes stopwords from transcript
    """

    def execute(self, option_dict=None):
        """
        :param option_dict: A dictionary consisting of
        "stopword_list": [a list of additional stopwords]
        :return: N/A
        """
        if (option_dict is not None) and ("stopword_list" in option_dict):
            stopword_list = option_dict["stopword_list"]
        else:
            stopword_list = []

        if self.transcript.df_transcript is not None:
            # Check language to get the correct stopwords
            # Add upper and lowercase stopwords to work with WordCaseCleaner in Pipeline
            if self.transcript.get_language()[0] == "DE":
                stopword_list += stopwords.words('german') + [word.upper() for word in stopwords.words('german')]
            else:
                stopword_list += stopwords.words('english') + [word.upper() for word in stopwords.words('english')]
            self.transcript.df_transcript["Transcript"] = self.transcript.df_transcript["Transcript"].apply(
                lambda x: " ".join([item for item in x.split(" ") if item not in stopword_list]))


if __name__ == "__main__":
    print("All is good!")



"""
Loader module
Load text from your data files into a format that allows for analysis
"""

import pandas as pd


def load_interview_transcript(transcript):
    """Reads interview transcript csv file as a dataframe.
    """
    transcript_df = pd.read_csv(transcript)
    return transcript_df


if __name__ == "__main__":
    print("All good!")



from eazynlp.base.transcript import Transcript


class Pipeline:
    """
    Superclass that defines basic properties and methods of pipelines
    """
    def __init__(self, transcript: Transcript, steps=None):
        """
        :param transcript: Transcript of type Transcript object
        :param steps: Steps in pipeline (default set to None)
        """
        if steps is None:
            steps = []
        self.transcript = transcript
        self.steps = steps

    def set_steps(self, steps):
        """
        :param steps: Steps in pipeline
        :return: N/A
        """
        self.steps = steps

    def execute(self):
        """
        Shell method for executing pipeline
        Expects each step in step list to contain a tuple
        of the form (step, dictionary of options for that step)
        :return: N/A
        """
        # for step in self.steps:
        #    step(self.transcript).execute()
        for (step, option_dict) in self.steps:
            step(self.transcript).execute(option_dict)


if __name__ == "__main__":
    print("All good!")



from base.transcript import Transcript
from tests.test_utils import get_sample_data_filepath
from tests.test_utils import get_sample_metadata_filepath
from text_miner.analysis import Analysis
from visualizer.visualizer import Visualization
from wordcloud import WordCloud  # , STOPWORDS
from nltk.corpus import stopwords
from nltk.tokenize import RegexpTokenizer
import matplotlib.pyplot as plt


class WordCountAnalysis(Analysis, Visualization):
    """
    Analysis class that generates word count statistics
    for a transcript.
    """

    # def __init__(self, wordcloud=None):
    #    super().__init__()
    #    self.wordcloud = wordcloud

    def generate_analysis_result(self):
        """
        Shell method for generating analysis result in child class
        """
        transcript = self.transcript
        df_text = transcript.load_df_text()
        wordcount_result = df_text["Transcript"].str.split(expand=True).stack().value_counts()
        return wordcount_result

    def save_analysis_result(self, target_filepath):
        """
        Shell method for saving analysis result in child class
        to a file
        :param target_filepath: Target filepath to save analysis result
        """
        res.to_csv(
            path_or_buf=target_filepath,
            sep=",",
            header=False,
            index=True
        )

    def show_visualization(self, visual_type=None):
        """
        Shell method for showing visualization in child class
        :param visual_type: Type of visualization
        """
        transcript = self.transcript
        df_text = transcript.load_df_text()
        text = df_text["Transcript"].iloc[1:].to_string()
        tokenizer = RegexpTokenizer(r'\w+')
        text = tokenizer.tokenize(text)
        text = " ".join(str(item) for item in text)
        if visual_type == "wordcloud":
            if transcript.get_language()[0] == "DE":
                stopword_set = set(stopwords.words('german'))
            else:
                stopword_set = set(stopwords.words('english'))
            # self.wordcloud = WordCloud(
            wordcloud = WordCloud(
                width=1500,
                height=1000,
                background_color='black',
                stopwords=stopword_set
            ).generate(text)
            plt.figure(
                figsize=(40, 30),
                facecolor='k',
                edgecolor='k')
            # plt.imshow(self.wordcloud, interpolation='bilinear')
            plt.imshow(wordcloud, interpolation='bilinear')
            plt.axis('off')
            plt.tight_layout(pad=0)
            plt.show()

    def save_visualization(self, visual_type, target_filepath=None):
        """
        Shell method for saving analysis result in child class
        to a file
        :param visual_type: Type of visualization
        :param target_filepath: Target filepath to save visualization
        """
        # if visual_type == "wordcloud": self.wordcloud.to_file(target_filepath)
        pass


if __name__ == "__main__":
    t = Transcript()

    t.set_metadata_filepath(get_sample_metadata_filepath())
    lang = t.get_language()
    # print(lang[0])

    t.set_transcript_filepath(get_sample_data_filepath())
    sample_interview = t.load_df_text()
    print(sample_interview.head())

    a = WordCountAnalysis(t)
    # print(a.transcript.transcript_filepath)
    res = a.generate_analysis_result()
    a.save_analysis_result("wordcount_analysis.csv")
    """res.to_csv(
        path_or_buf="wordcount_analysis.csv",
        sep=",",
        header=False,
        index=True
    )"""
    a.show_visualization(visual_type="wordcloud")


from eazynlp.base.transcript import Transcript


class InterviewTranscript(Transcript):
    """
    Child class of transcript for interview transcript objects
    """

    def __init__(self, transcript_filepath=None, metadata_filepath=None):
        super(InterviewTranscript, self).__init__(transcript_filepath, metadata_filepath)


if __name__ == "__main__":
    print("All good!")



from eazynlp.base.transcript import Transcript
from eazynlp.base.pipeline import Pipeline
from eazynlp.text_cleaner.punctuation_cleaner import PunctuationCleaner
from eazynlp.text_cleaner.stopword_cleaner import StopwordCleaner
from eazynlp.text_cleaner.wordcase_cleaner import WordCaseCleaner
from eazynlp.text_miner.analysis import Analysis
from eazynlp.visualizer.visualizer import Visualization
from wordcloud import WordCloud
import matplotlib.pyplot as plt


class WordCountAnalysis(Analysis, Visualization):
    """
    Analysis class that generates word count statistics
    for a transcript.
    """

    def generate_analysis_result(self):
        """
        Shell method for generating analysis result in child class
        """
        transcript = self.transcript
        Pipeline(transcript,
                 [
                     (WordCaseCleaner, {"to_lower": True}),
                     (PunctuationCleaner, {}),
                     (StopwordCleaner, {"stopword_list": ["ja"]})
                 ]
                 ).execute()
        text = transcript.df_transcript["Transcript"]
        wordcount_result = text.str.split(expand=True).stack().value_counts()
        return wordcount_result

    def save_analysis_result(self, target_filepath):
        """
        Shell method for saving analysis result in child class
        to a file
        :param target_filepath: Target filepath to save analysis result
        """
        wordcount_result = self.generate_analysis_result()
        wordcount_result.to_csv(
            path_or_buf=target_filepath,
            sep=",",
            header=False,
            index=True
        )

    def show_visualization(self, visual_type=None):
        """
        Shell method for showing visualization in child class
        :param visual_type: Type of visualization
        """
        transcript = self.transcript
        Pipeline(transcript,
                 [
                     (WordCaseCleaner, {"to_lower": True}),
                     (PunctuationCleaner, {}),
                     (StopwordCleaner, {"stopword_list": ["ja"]})
                 ]
                 ).execute()
        text = transcript.df_transcript["Transcript"].str.cat(sep=" ")
        if visual_type == "wordcloud":
            wordcloud = WordCloud(
                width=1500,
                height=1000,
                background_color='black'
            ).generate(text)
            plt.figure(
                figsize=(40, 30),
                facecolor='k',
                edgecolor='k')
            plt.imshow(wordcloud, interpolation='bilinear')
            plt.axis('off')
            plt.tight_layout(pad=0)
            plt.show()

    def save_visualization(self, visual_type, target_filepath=None):
        """
        Shell method for saving analysis result in child class
        to a file
        :param visual_type: Type of visualization
        :param target_filepath: Target filepath to save visualization
        """
        pass


if __name__ == "__main__":
    print("All good!")



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="eazynlp",
    version="0.0.7",
    author="Info CodeBrew",
    author_email="info.codebrew@gmail.com",
    description="NLP made easy for researchers",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/tumeri/eazynlp",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    install_requires=["matplotlib", "nltk", "pandas", "wordcloud"],
    python_requires='>=3.6'
)



class Visualization:
    """
    Superclass that defines the basic properties and methods
    supported by child classes that implement specific visualizations.
    """
    visualization = None

    def __init__(self):
        pass

    def show_visualization(self, visual_type):
        """
        Shell method for showing visualization in child class
        :param visual_type: Type of visualization
        """
        pass

    def save_visualization(self, visual_type, target_filepath=None):
        """
        Shell method for saving analysis result in child class
        to a file
        :param visual_type: Type of visualization
        :param target_filepath: Target filepath to save visualization
        """
        pass



from eazynlp.base.transcript import Transcript
from eazynlp.text_cleaner.cleaner import Cleaner


class WordCaseCleaner(Cleaner):
    """
    A subclass of Cleaner superclass that converts transcript to lower/uppercase
    """

    def execute(self, option_dict=None):
        """
        :param option_dict: A dictionary consisting of
        "to_lower": True (convert transcript to lowercase)
        or False (convert transcript to uppercase)
        :return: N/A
        """
        if (option_dict is not None) and ("to_lower" in option_dict):
            to_lower = option_dict["to_lower"]
        else:
            to_lower = True

        if self.transcript.df_transcript is not None:
            if to_lower:
                self.transcript.df_transcript["Transcript"] = self.transcript.df_transcript["Transcript"].str.lower()
            else:
                self.transcript.df_transcript["Transcript"] = self.transcript.df_transcript["Transcript"].str.upper()


if __name__ == "__main__":
    print("All is good!")
